<?xml version='1.0' encoding='UTF-8'?>
<!DOCTYPE gui SYSTEM 'kpartgui.dtd'>
<gui name="kxmlguiwindowtest" version="1">
 <ToolBar name="mainToolBar">
  <text>Main Toolbar</text>
  <Action name="test"/>
 </ToolBar>
 <ToolBar name="otherToolBar">
  <text>Other Toolbar</text>
  <Action name="file_quit"/>
 </ToolBar>
 <MenuBar>
  <Menu name="file">
   <Action name="test"/>
   <Action name="file_quit"/>
  </Menu>
 </MenuBar>
 <ActionProperties scheme="Default">
  <Action name="help_report_bug" priority="128"/>
 </ActionProperties>
</gui>
