/*
    SPDX-FileCopyrightText: 2017 Alexander Potashev <aspotashev@gmail.com>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#ifndef KTOOLBARHELPER_P_H
#define KTOOLBARHELPER_P_H

#include <QDomDocument>

namespace KToolbarHelper
{

QString i18nToolBarName(const QDomElement &element);

}

#endif // KTOOLBARHELPER_P_H
